﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Phidget22.ExampleUtils
{
	public partial class DataGraphBox : Form
	{

		private double chartTime = 20;
		private bool pause = false;

		private String filename = "";

		List<double[]> allDataPoints = new List<double[]>();

		System.IO.StreamWriter fs;

		public DataGraphBox(Form owner, String title)
		{
			InitializeComponent();
			this.Owner = owner;
			IntPtr dummy = this.Handle; //Force the form to load before it's shown so we can call invoke on it.
			graph.ChartAreas["ChartArea1"].AxisX.LabelStyle.Format = "hh:mm:ss tt";
			graph.Titles[0].Text = title;
			this.Text = title;
			graph.ChartAreas["ChartArea1"].AxisX.Interval = 5;
			graph.ChartAreas["ChartArea1"].AxisX.IntervalType = System.Windows.Forms.DataVisualization.Charting.DateTimeIntervalType.Seconds;
			ChartTimeLabel.Text = chartTime.ToString() + "s";
			ChartTimeTrackbar.Value = (int)(chartTime / 5);
			FileSizeLabel.Text = "";
			FileNameLabel.Text = "";
		}

		public void addData(double dataPoint)
		{
			if (pause)
				return;
			try {
				if (allDataPoints.Count > 5)
				{
					DateTime endTime = DateTime.FromOADate(allDataPoints.Last()[0]);
					double diff = (DateTime.Now).Subtract(endTime).TotalSeconds;
					double seconds;
					do
					{
						DateTime startTime = DateTime.FromOADate(allDataPoints.First()[0]);
						seconds = endTime.Subtract(startTime).TotalSeconds;
						if (seconds > chartTime)
						{
							allDataPoints.RemoveAt(0);
						}
					} while (seconds > chartTime);
				}
				DateTime nowTime = System.DateTime.Now;
				allDataPoints.Add(new double[] { nowTime.ToOADate(), dataPoint });

				if (filename != "")
				{
					//System.IO.StreamWriter fs = new System.IO.StreamWriter(filename, true);
					fs.WriteLine(nowTime.ToString("yyyy/MM/dd HH:mm:ss.fff") + "," + dataPoint.ToString());
					Int64 lengthKB = fs.BaseStream.Length / 1024;
					FileSizeLabel.Text = "File Size: " + lengthKB.ToString() + "kB";
					//fs.Close();
				}

				if (graph.Series[0].Points.Count > 5) {
					DateTime endTime = DateTime.FromOADate(graph.Series[0].Points.Last().XValue);
					double diff = (DateTime.Now).Subtract(endTime).TotalSeconds;
					if (DateTime.Now.Subtract(endTime).TotalSeconds < 0.02)
					{
						
						return;
					}
					double seconds;
					do
					{
						DateTime startTime = DateTime.FromOADate(graph.Series[0].Points.First().XValue);
						seconds = endTime.Subtract(startTime).TotalSeconds;
						if (seconds > chartTime)
						{
							graph.Series[0].Points.RemoveAt(0);
						}
					} while (seconds > chartTime);
				}

				
				graph.Series[0].Points.AddXY(System.DateTime.Now, dataPoint);
				graph.ResetAutoValues();
			} catch (Exception ex)
			{
				Console.WriteLine(ex);
			}
		}

		private void DataGraphBox_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (e.CloseReason == CloseReason.UserClosing)
			{
				this.Hide();
				e.Cancel = true;
				if (fs != null)
				{
					fs.Close();
				}
				filename = "";
				SaveButton.Text = "Save to CSV";
			}
		}

		private void trackBar1_Scroll(object sender, EventArgs e)
		{
			chartTime = ChartTimeTrackbar.Value * 5;
			ChartTimeLabel.Text = chartTime.ToString() + "s";
		}

		private void ClearButton_Click(object sender, EventArgs e)
		{
			graph.Series[0].Points.Clear();
			allDataPoints.Clear();
		}

		private void SaveButton_Click(object sender, EventArgs e)
		{
			if(filename != "")
			{
				filename = "";
				fs.Close();
				SaveButton.Text = "Save to CSV";
				return;
			}
				

			// Displays a SaveFileDialog so the user can save the Image
			// assigned to Button2.
			SaveFileDialog saveFileDialog1 = new SaveFileDialog();
			saveFileDialog1.Filter = "CSV file (.csv)|*.csv| All Files (.)|*.*";
			saveFileDialog1.Title = "Save to CSV File";
			saveFileDialog1.ShowDialog();

			// If the file name is not an empty string open it for saving.
			if (saveFileDialog1.FileName != "")
			{
				fs = new System.IO.StreamWriter(saveFileDialog1.OpenFile());
				filename = saveFileDialog1.FileName;
				FileNameLabel.Text = "File: " + filename;
				FileNameLabel.Location = new Point(this.Width - (FileNameLabel.Width + 30), FileNameLabel.Location.Y);

				fs.WriteLine("Date," + this.Text);
				foreach(double[] dataPoint in allDataPoints)
				{
					DateTime pointTime = DateTime.FromOADate(dataPoint[0]);
					fs.WriteLine(pointTime.ToString("yyyy/MM/dd HH:mm:ss.fff") + "," + dataPoint[1].ToString());
				}

				Int64 lengthKB = fs.BaseStream.Length / 1024;
				FileSizeLabel.Text = "File Size: " + lengthKB.ToString() + "kB";
				
				//fs.Close();

				if (pause) //Don't continue saving to a file if the data was paused
					filename = "";
				else
					SaveButton.Text = "Stop Saving";
			}
		}

		private void button1_Click(object sender, EventArgs e)
		{
			if (pause == false) {
				pause = true;
				PauseButton.Text = "Resume";
			}
			else
			{
				pause = false;
				PauseButton.Text = "Pause";
			}
		}
	}
}
